/**
 * Copyright (c) 2009 Motorola.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Aragao (Motorola) - Initial version
 */

package org.eclipse.mtj.internal.core.build.export.states;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.mtj.core.project.IMTJProject;
import org.eclipse.mtj.core.project.midp.IMidletSuiteProject;
import org.eclipse.mtj.core.project.runtime.MTJRuntime;
import org.eclipse.mtj.internal.core.build.export.AntennaExportException;
import org.eclipse.mtj.internal.core.project.midp.MidletSuiteFactory;
import org.eclipse.mtj.internal.core.statemachine.StateMachine;
import org.eclipse.mtj.internal.core.util.Utils;
import org.eclipse.mtj.internal.core.util.xml.XMLUtils;
import org.eclipse.osgi.util.NLS;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * CreateAntGeneratePropertiesState creates property files from the Application
 * Descriptor file for the build export.
 * 
 * @author David Aragao
 * @since 1.0
 */
public class CreateAntBuildFilesFilterState extends AbstractCreateAntTaskState {

	/**
	 * Creates a {@link CreateAntBuildTaskState} instance bound to the specified
	 * state machine in order to create build target for the specified project.
	 * 
	 * @param machine
	 *            bound {@link StateMachine} instance.
	 * @param project
	 *            target {@link IMidletSuiteProject} instance.
	 * @param _document
	 *            target {@link Document}.
	 */
	public CreateAntBuildFilesFilterState(StateMachine machine,
			IMidletSuiteProject project, Document _document,String buildFolder, String buildFile) {
		super(machine, project, _document, buildFolder, buildFile);
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.mtj.internal.core.build.export.states.AbstractCreateAntTaskState
	 * #onEnter(org.eclipse.mtj.core.project.runtime.MTJRuntime)
	 */
	protected void onEnter(MTJRuntime runtime) throws AntennaExportException {
		Document document = getDocument();
		Element root = document.getDocumentElement();
		String configName = getFormatedName(runtime.getName());
		IProject project = getMidletSuiteProject().getProject();
		
		Element filter = XMLUtils.createTargetElement(document, root, 
				NLS.bind("buildFilesFilter-{0}",configName), getDependencies(runtime)); //$NON-NLS-1$
		this.buildFilesFilter(project, document, filter, runtime);
		
	}
	
	/**
	 * Creates a task to create the specified project classes.
	 * 
	 * @param _project
	 *            target project
	 * @param _document
	 *            target document
	 * @param _root
	 *            document root
	 * @param _runtime
	 *            current runtime
	 * @throws AntennaExportException
	 *             Any error occurs
	 */
	private void buildFilesFilter(IProject _project, Document _document,
			Element _root, MTJRuntime _runtime)throws AntennaExportException{
		IJavaProject javaProject = JavaCore.create(_project);
		if (javaProject == null) {
			return;
		}

		IMTJProject suiteProject = MidletSuiteFactory
				.getMidletSuiteProject(javaProject);
		if (suiteProject == null) {
			return;
		}
		String configName = getFormatedName(_runtime.getName());
		Element buildFilesFilter = _document.createElement("buildFilesFilter"); //$NON-NLS-1$
		_root.appendChild(buildFilesFilter);
		
		buildFilesFilter.setAttribute("dirclasses", NLS //$NON-NLS-1$
				.bind(
						"{0}/{1}/{2}/classes/", new String[] { getBuildFolder(), configName, //$NON-NLS-1$
								getFormatedName(_project
										.getName()) }));
		buildFilesFilter.setAttribute("dirresources", NLS //$NON-NLS-1$
				.bind(
						"{0}/{1}/{2}/resources/", new String[] { getBuildFolder(), configName, //$NON-NLS-1$
								getFormatedName(_project
										.getName()) }));
		buildFilesFilter.setAttribute("buildProperties", "${basedir}/build.properties");  //$NON-NLS-1$//$NON-NLS-2$
		buildFilesFilter.setAttribute("device", configName); //$NON-NLS-1$
		
		IResource[] sources = Utils.getSourceFolders(suiteProject
				.getJavaProject());
		
		for (IResource source : sources) {
			Element fileset = _document.createElement("fileset"); //$NON-NLS-1$
			fileset.setAttribute("dir", NLS.bind("..{0}", source //$NON-NLS-1$ //$NON-NLS-2$
					.getFullPath()));
			//fileset.setAttribute("includes", "**/**.java"); //$NON-NLS-1$ //$NON-NLS-2$
			buildFilesFilter.appendChild(fileset);
		}
	}
}
